// =============================================================================
//	Copyright (C) 2009-2012, Heartland Data Co, Ltd.  All Rights Reserved.
//	
//	Title:	Asynchronous Bus Driver
//	
// =============================================================================

// =============================================================================
// 	Please customize the code for your environment.
// =============================================================================

//------------------------------------------------------------------------------
//	Desc:	Header for Port Controll
//------------------------------------------------------------------------------
#include "include.h"

//------------------------------------------------------------------------------
//  Macro:	DT_UINT
//	Desc:	Please change Test Point argument type for DT10 Project setting.
//------------------------------------------------------------------------------
#define	DT_UINT unsigned int

//------------------------------------------------------------------------------
//  Macro:	DT_USE_FUNCTION
//	Desc:	Please set 1 when you use function call.
//------------------------------------------------------------------------------
#define	DT_USE_FUNCTION 	0

//------------------------------------------------------------------------------
//	Macro:	DT_INLINE
//	Desc: 	Please use "static" instead of "inline" if "inline" cannot be used.
//------------------------------------------------------------------------------
#define	DT_INLINE	inline
//#define	DT_INLINE	_inline		// for Windows
//#define	DT_INLINE	static		// when "inline" cannot be used

//------------------------------------------------------------------------------
//	Func:	enterCritical
//	Desc:	Please describe the code to enter Critical Section.
//------------------------------------------------------------------------------
DT_INLINE void enterCritical(void)
{
	// ex)	DI();
	// 
	taskENTER_CRITICAL();
}

//------------------------------------------------------------------------------
//	Func:	exitCritical
//	Desc:	Please describe the code to exit Critical Section.
//------------------------------------------------------------------------------
DT_INLINE void exitCritical(void)
{
	// ex)	EI();
	// 
	taskEXIT_CRITICAL();
}

//------------------------------------------------------------------------------
//	Func: _TP_BusOutDrv
//	Desc: Test Point Output Function
//
//------------------------------------------------------------------------------
#if (DT_USE_FUNCTION == 1)
DT_INLINE void _TP_BusOutDrv( DT_UINT addr, DT_UINT dat )
{
	*((volatile short *)addr) = dat;
}
#else
#define	_TP_BusOutDrv( addr, dat )					\
	*((volatile short *)(addr)) = (dat);
#endif

// =============================================================================
// 	Don't change the code from here as possible.
// =============================================================================

//------------------------------------------------------------------------------
//	Desc:	Test Point IDs
//------------------------------------------------------------------------------
#define		DT_VARIABLE_BIT			0x02
#define		DT_EVTTRG_BIT			0x08

//------------------------------------------------------------------------------
//	Func: _TP_BusOut
//	Desc: Called by Test Point
//------------------------------------------------------------------------------
#if DT_USE_FUNCTION
void _TP_BusOut( DT_UINT addr, DT_UINT dat )
{
	_TP_BusOutDrv( addr, dat );
}
#endif

//------------------------------------------------------------------------------
//	Func: _TP_MemoryOutput
//	Desc: Called by Variable Test Point
//------------------------------------------------------------------------------
void _TP_MemoryOutput( DT_UINT addr, DT_UINT dat, void *value, DT_UINT size )
{
	DT_UINT	i;
	unsigned char *p = value;
#if (DT_USE_FUNCTION == 0)
	DT_UINT base = addr & 0xfff00000;
#endif
	enterCritical();
	_TP_BusOutDrv( addr | DT_VARIABLE_BIT, dat );
	if( size >= 256 ) size = 256;
	for( i = 0; i < size; i += 2 ){
		dat = p[i] ;
		if( (i + 1) < size ){
			dat |= p[i+1] << 8;
		}
		addr = DT_VARIABLE_BIT ;		// Variable Bit
		if( i == 0 ){
			addr |= size << 8 ;			// Size
		}
#if (DT_USE_FUNCTION == 0)
		addr |= base;
#endif
		_TP_BusOutDrv( addr, dat );
	}
	exitCritical();
}

//------------------------------------------------------------------------------
//	Func: _TP_EventTrigger
//	Desc: Called by Event Trigger Test Point
//------------------------------------------------------------------------------
void _TP_EventTrigger( DT_UINT addr, DT_UINT dat )
{
	_TP_BusOutDrv( addr | DT_EVTTRG_BIT, ((unsigned short)(dat&0x0FFF))<<4 );
}

// =============================================================================
